@@ -1,17 +1,16 @@
-
 """The tsdat.qc package provides the classes that the data pipeline
 uses to manage quality control/quality assurance for the dataset.  This
 includes the infrastructure to run quality tests and handle failures,
-as well specific checkers and handlers that can be specified in the 
+as well specific checkers and handlers that can be specified in the
 pipeline config file.
 
- We warmly welcome community contribututions to increase this default list.
+We warmly welcome community contribututions to increase this default list.
 """
 from .qc import QualityManagement, QualityManager
 from .checkers import (
-    QualityChecker, 
+    QualityChecker,
     CheckWarnMax,
-    CheckFailMax, 
+    CheckFailMax,
     CheckFailMin,
     CheckMax,
     CheckMin,
@@ -20,7 +19,7 @@ from .checkers import (
     CheckValidDelta,
     CheckValidMax,
     CheckValidMin,
-    CheckWarnMin   
+    CheckWarnMin,
 )
 from .handlers import (
     QualityHandler,
@@ -28,5 +27,5 @@ from .handlers import (
     FailPipeline,
     RecordQualityResults,
     RemoveFailedValues,
-    SendEmailAWS
-)
\ No newline at end of file
+    SendEmailAWS,
+)
