@@ -1,14 +1,11 @@
-import mimetypes
-
-import datetime
-import matplotlib.pyplot as plt
+import os
+import act
 import numpy as np
 import pandas as pd
-import os
 import xarray as xr
-from typing import List, Dict, Tuple, Union, Optional
+import matplotlib.pyplot as plt
 
-import act
+from typing import List, Dict, Tuple, Optional
 from tsdat.constants import ATTS
 
 # Note that we can't use these in the type hints because
@@ -16,17 +13,14 @@ from tsdat.constants import ATTS
 # from tsdat.config import Config, VariableDefinition
 
 
-mimetypes.init()
-
-
-class DSUtil:  
+class DSUtil:
     """
     Provides helper functions for xarray.Dataset
     """
 
     @staticmethod
     def record_corrections_applied(ds: xr.Dataset, variable: str, correction: str):
-        """Records a description of a correction made to a variable to the 
+        """Records a description of a correction made to a variable to the
         corrections_applied corresponding attribute.
 
         :param ds: Dataset containing the corrected variable
@@ -35,13 +29,13 @@ class DSUtil:
         :type variable: str
         :param correction: A description of the correction
         :type correction: str
-        """        
+        """
         corrections = ds[variable].attrs.get(ATTS.CORRECTIONS_APPLIED, [])
         corrections.append(correction)
         ds[variable].attrs[ATTS.CORRECTIONS_APPLIED] = corrections
 
     @staticmethod
-    def datetime64_to_string(datetime64: Union[np.ndarray, np.datetime64]) -> Tuple[str, str]:
+    def datetime64_to_string(datetime64: np.datetime64) -> Tuple[str, str]:
         """Convert a datetime64 object to formated string.
 
         :param datetime64: The datetime64 object
@@ -49,8 +43,8 @@ class DSUtil:
         :return: A tuple of strings representing the formatted date.  The first string is
             the day in 'yyyymmdd' format.  The second string is the time in 'hhmmss' format.
         :rtype: Tuple[str, str]
-        """        
-        datetime = act.utils.datetime64_to_datetime()[0]
+        """
+        datetime = act.utils.datetime64_to_datetime(datetime64)[0]
         return datetime.strftime("%Y%m%d"), datetime.strftime("%H%M%S")
 
     @staticmethod
@@ -60,10 +54,10 @@ class DSUtil:
 
         :param variable_data: ndarray of variable data
         :type variable_data: np.ndarray
-        :return: An ndarray of the same shape, with time values converted to 
+        :return: An ndarray of the same shape, with time values converted to
             long timestamps (e.g., int64)
         :rtype: np.ndarray
-        """        
+        """
         return variable_data.astype(pd.Timestamp).astype(np.int64)
 
     @staticmethod
@@ -73,18 +67,16 @@ class DSUtil:
 
         :param ds: The data as an xarray dataset; defaults to None
         :type ds: xr.Dataset, optional.
-        :param config: The Config object used to assist reading time data from 
+        :param config: The Config object used to assist reading time data from
             the raw_dataset; defaults to None.
         :type config: Config, optional
         :return: The datastream name
         :rtype: str
-        """        
-        assert(ds is not None or config is not None)
-        if ds is not None and "datastream_name" in ds.attrs:
+        """
+        assert ds or config
+        if ds and "datastream_name" in ds.attrs:
             return ds.attrs["datastream_name"]
-        elif config :
-            return config.dataset_definition.datastream_name
-        return None
+        return config.dataset_definition.attrs.get("datastream_name")
 
     @staticmethod
     def get_end_time(ds: xr.Dataset) -> Tuple[str, str]:
@@ -97,89 +89,9 @@ class DSUtil:
             the last time point in the dataset.
         :rtype: Tuple[str, str]
         """
-        time64 = np.min(ds['time'].data)
+        time64 = np.max(ds["time"].data)
         return DSUtil.datetime64_to_string(time64)
 
-    @staticmethod
-    def get_fail_max(ds: xr.Dataset, variable_name: str):     
-        """Get the max value from the fail_range attribute
-        for the given variable.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: The max value of the fail_range attribute or 
-            None if not defined
-        :rtype: same data type of the variable (int, float, etc.)
-            or None
-        """        
-        fail_max = None
-        fail_range = ds[variable_name].attrs.get(ATTS.FAIL_RANGE, None)
-        if fail_range is not None:
-            fail_max = fail_range[-1]
-        return fail_max
-
-    @staticmethod
-    def get_fail_min(ds: xr.Dataset, variable_name):
-        """Get the min value from the fail_range attribute
-        for the given variable.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: The min value of the fail_range attribute or 
-            None if not defined
-        :rtype: same data type of the variable (int, float, etc.)
-            or None
-        """     
-        fail_min = None
-        fail_range = ds[variable_name].attrs.get(ATTS.FAIL_RANGE, None)
-        if fail_range is not None:
-            fail_min = fail_range[0]
-        return fail_min
-
-    @staticmethod
-    def get_valid_max(ds: xr.Dataset, variable_name):
-        """Get the max value from the valid_range attribute
-        for the given variable.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: The max value of the valid_range attribute or 
-            None if not defined
-        :rtype: same data type of the variable (int, float, etc.)
-            or None
-        """     
-        valid_max = None
-        valid_range = ds[variable_name].attrs.get(ATTS.VALID_RANGE, None)
-        if valid_range is not None:
-            valid_max = valid_range[-1]
-        return valid_max
-
-    @staticmethod
-    def get_valid_min(ds: xr.Dataset, variable_name):
-        """Get the min value from the valid_range attribute
-        for the given variable.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: The min value of the valid_range attribute or 
-            None if not defined
-        :rtype: same data type of the variable (int, float, etc.)
-            or None
-        """     
-        valid_min = None
-        valid_range = ds[variable_name].attrs.get(ATTS.VALID_RANGE, None)
-        if valid_range is not None:
-            valid_min = valid_range[0]
-        return valid_min
-
     @staticmethod
     def get_fill_value(ds: xr.Dataset, variable_name: str):
         """Get the value of the _FillValue attribute
@@ -193,7 +105,7 @@ class DSUtil:
             if it is not defined
         :rtype: same data type of the variable (int, float, etc.)
             or None
-        """        
+        """
         return ds[variable_name].attrs.get(ATTS.FILL_VALUE, None)
 
     @staticmethod
@@ -205,18 +117,8 @@ class DSUtil:
         :type ds: xr.Dataset
         :return: List of non-qc data variable names
         :rtype: List[str]
-        """        
-        varnames = []
-
-        def exclude_qc(variable_name):
-            if variable_name.startswith('qc_'):
-                return False
-            else:
-                return True
-
-        varnames = filter(exclude_qc, list(ds.data_vars.keys()))
-
-        return varnames
+        """
+        return [var for var in ds.data_vars.keys() if not var.startswith("qc_")]
 
     @staticmethod
     def get_raw_end_time(raw_ds: xr.Dataset, time_var_definition) -> Tuple[str, str]:
@@ -227,22 +129,21 @@ class DSUtil:
 
         :param raw_ds: A raw dataset (not standardized)
         :type raw_ds: xr.Dataset
-        :param time_var_definition: The 'time' variable definition from the 
+        :param time_var_definition: The 'time' variable definition from the
             pipeline config
         :type time_var_definition: VariableDefinition
         :return: A tuple of strings representing the last time data point
-            in the dataset.  The first string is the day in 'yyyymmdd' format.  
+            in the dataset.  The first string is the day in 'yyyymmdd' format.
             The second string is the time in 'hhmmss' format.
         :rtype: Tuple[str, str]
-        """        
+        """
         time_var_name = time_var_definition.get_input_name()
-        time_data = raw_ds[time_var_name].values
+        time_data = raw_ds[time_var_name].data
 
         time64_data = time_var_definition.run_converter(time_data)
 
         end_datetime64 = np.max(time64_data)
-        end: datetime.datetime = act.utils.datetime64_to_datetime(end_datetime64)[0]
-        return end.strftime("%Y%m%d"), end.strftime("%H%M%S")
+        return DSUtil.datetime64_to_string(end_datetime64)
 
     @staticmethod
     def get_raw_start_time(raw_ds: xr.Dataset, time_var_definition) -> Tuple[str, str]:
@@ -253,22 +154,21 @@ class DSUtil:
 
         :param raw_ds: A raw dataset (not standardized)
         :type raw_ds: xr.Dataset
-        :param time_var_definition: The 'time' variable definition from the 
+        :param time_var_definition: The 'time' variable definition from the
             pipeline config
         :type time_var_definition: VariableDefinition
         :return: A tuple of strings representing the first time data point
-            in the dataset.  The first string is the day in 'yyyymmdd' format.  
+            in the dataset.  The first string is the day in 'yyyymmdd' format.
             The second string is the time in 'hhmmss' format.
         :rtype: Tuple[str, str]
-        """        
+        """
         time_var_name = time_var_definition.get_input_name()
-        time_data = raw_ds[time_var_name].values
+        time_data = raw_ds[time_var_name].data
 
         time64_data = time_var_definition.run_converter(time_data)
 
         start_datetime64 = np.min(time64_data)
-        start: datetime.datetime = act.utils.datetime64_to_datetime(start_datetime64)[0]
-        return start.strftime("%Y%m%d"), start.strftime("%H%M%S")
+        return DSUtil.datetime64_to_string(start_datetime64)
 
     @staticmethod
     def get_coordinate_variable_names(ds: xr.Dataset) -> List[str]:
@@ -278,33 +178,8 @@ class DSUtil:
         :type ds: xr.Dataset
         :return: List of coordinate variable names
         :rtype: List[str]
-        """        
-        return list(ds.coords.keys())
-
-    @staticmethod
-    def get_shape(ds: xr.Dataset, variable_name: str) -> Tuple[List[str], List[int]]:
-        """Get the shape of a variable's data.  Convenience 
-        method to provide access to dimension names and their
-        lengths in one call.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: A tuple where the first value is an array of the dimension
-            names belonging to this variable and the second value is an array
-            of the corresponding lengths of those dimensions.
-        :rtype: Tuple[List[str], List[int]]
         """
-        var = ds.get(variable_name)
-        dims = []
-        lengths = []
-
-        for dim in var.sizes:
-            dims.append(dim)
-            lengths.append(var.sizes[dim])
-
-        return dims, lengths
+        return list(ds.coords.keys())
 
     @staticmethod
     def get_start_time(ds: xr.Dataset) -> Tuple[str, str]:
@@ -314,47 +189,12 @@ class DSUtil:
         :param ds: A standardized dataset
         :type ds: xr.Dataset
         :return: A tuple of strings representing the first time data point
-            in the dataset.  The first string is the day in 'yyyymmdd' format.  
+            in the dataset.  The first string is the day in 'yyyymmdd' format.
             The second string is the time in 'hhmmss' format.
         :rtype: Tuple[str, str]
-        """        
-        time64 = np.min(ds['time'].data)
-        start = act.utils.datetime64_to_datetime(time64)[0]
-        return start.strftime("%Y%m%d"), start.strftime("%H%M%S")
-
-    @staticmethod
-    def get_timestamp(dt64: np.datetime64):
-        """Convert a datetime64 value into a long integer timestamp
-        :param dt64: datetime64 object
-        :return: timestamp in seconds since 1970-01-01T00:00:00Z
-        :rtype: int
         """
-        ts = int((dt64 - np.datetime64('1970-01-01T00:00:00Z')) / np.timedelta64(1, 's'))
-        return ts
-
-    @staticmethod
-    def get_variables_with_dimension(ds: xr.Dataset, dim_name: str, include_qc=False) -> List[str]:
-        """Find all variables dimensioned by the given dim.
-        Note that this method will only get data variables, NOT coordinate
-        variables.
-
-        :param ds: A dataset
-        :type ds: xr.Dataset
-        :param dim_name: Dimension name
-        :type dim_name: str
-        :param include_qc: Should qc variables be included, defaults to False
-        :type include_qc: bool, optional
-        :return: A list of all variable names that have that dimension
-        :rtype: List[str]
-        """        
-        variable_names: List[str] = []
-        for variable_name in ds.data_vars:
-            if include_qc or not variable_name.startswith('qc_'):
-                variable = ds.get(variable_name)
-                for dim in variable.sizes:
-                    if dim == dim_name:
-                        variable_names.append(variable_name)
-        return variable_names
+        time64 = np.min(ds["time"].data)
+        return DSUtil.datetime64_to_string(time64)
 
     @staticmethod
     def get_metadata(ds: xr.Dataset) -> Dict:
@@ -367,72 +207,16 @@ class DSUtil:
         :type ds: xr.Dataset
         :return: A dictionary of global & variable attributes
         :rtype: Dict
-        """        
+        """
         attributes = ds.attrs
         variables = {var_name: ds[var_name].attrs for var_name in ds.variables}
         metadata = {"attributes": attributes, "variables": variables}
         return metadata
 
     @staticmethod
-    def get_warn_max(ds: xr.Dataset, variable_name):
-        """Get the max value from the warn_range attribute
-        for the given variable.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: The max value of the warn_range attribute or 
-            None if not defined
-        :rtype: same data type of the variable (int, float, etc.)
-            or None
-        """   
-        warn_max = None
-        warn_range = ds[variable_name].attrs.get(ATTS.WARN_RANGE, None)
-        if warn_range is not None:
-            warn_max = warn_range[-1]
-        return warn_max
-
-    @staticmethod
-    def get_warn_min(ds: xr.Dataset, variable_name):
-        """Get the min value from the warn_range attribute
-        for the given variable.
-
-        :param ds: The dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: The min value of the warn_range attribute or 
-            None if not defined
-        :rtype: same data type of the variable (int, float, etc.)
-            or None
-        """   
-        warn_min = None
-        warn_range = ds[variable_name].attrs.get(ATTS.WARN_RANGE, None)
-        if warn_range is not None:
-            warn_min = warn_range[0]
-        return warn_min
-
-    @staticmethod
-    def is_coord_var(ds: xr.Dataset, variable_name: str) -> bool:
-        """Determine if given variable is a coordinate variable
-        for a dimension.
-
-        :param ds: A dataset
-        :type ds: xr.Dataset
-        :param variable_name: A variable in the dataset
-        :type variable_name: str
-        :return: True if the variable is a coordinate variable
-        :rtype: bool
-        """        
-        for dim in ds.coords.dims.keys():
-            if variable_name == dim:
-                return True
-
-        return False
-
-    @staticmethod
-    def plot_qc(ds: xr.Dataset, variable_name: str, filename: str=None):       
+    def plot_qc(
+        ds: xr.Dataset, variable_name: str, filename: str = None, **kwargs
+    ) -> act.plotting.TimeSeriesDisplay:
         """Create a QC plot for the given variable.  This is based on the ACT library:
         https://arm-doe.github.io/ACT/source/auto_examples/plot_qc.html#sphx-glr-source-auto-examples-plot-qc-py
 
@@ -442,7 +226,7 @@ class DSUtil:
 
         TODO: Depending on use cases, we will likely add more arguments to be able to quickly produce
         the most common types of QC plots.
-        
+
         :param ds: A dataset
         :type ds: xr.Dataset
         :param variable_name: The variable to plot
@@ -450,8 +234,10 @@ class DSUtil:
         :param filename: The filename for the image.  Saves the plot as this filename if provided.
         :type filename: str, optional
         """
-
-        display = act.plotting.TimeSeriesDisplay(ds, figsize=(15, 10), subplot_shape=(2,))
+        datastream_name = DSUtil.get_datastream_name(ds=ds)
+        display = act.plotting.TimeSeriesDisplay(
+            ds, subplot_shape=(2,), ds_name=datastream_name, **kwargs
+        )
 
         # Plot temperature data in top plot
         display.plot(variable_name, subplot_index=(0,))
@@ -459,14 +245,15 @@ class DSUtil:
         # Plot QC data
         display.qc_flag_block_plot(variable_name, subplot_index=(1,))
 
-        # Either display or save the plot, depending upon the parameters passed
         if filename:
             plt.savefig(filename)
-        else:
-            plt.show()
+
+        return display
 
     @staticmethod
-    def get_plot_filename(dataset: xr.Dataset, plot_description: str, extension: str) -> str:
+    def get_plot_filename(
+        dataset: xr.Dataset, plot_description: str, extension: str
+    ) -> str:
         """Returns the filename for a plot according to MHKIT-Cloud Data
         standards. The dataset is used to determine the datastream_name and
         start date/time. The standards dictate that a plot filename should
@@ -475,14 +262,14 @@ class DSUtil:
         :param dataset: The dataset from which the plot data is drawn from.
             This is used to collect the datastream_name and start date/time.
         :type dataset: xr.Dataset
-        :param plot_description: The description of the plot. Should be as 
+        :param plot_description: The description of the plot. Should be as
             brief as possible and contain no spaces. Underscores may be used.
         :type plot_description: str
         :param extension: The file extension for the plot.
         :type extension: str
         :return: The standardized plot filename.
         :rtype: str
-        """        
+        """
         datastream_name = DSUtil.get_datastream_name(dataset)
         date, time = DSUtil.get_start_time(dataset)
         return f"{datastream_name}.{date}.{time}.{plot_description}.{extension}"
@@ -501,7 +288,7 @@ class DSUtil:
         :type file_extension: str, optional
         :return: The base filename of the dataset.
         :rtype: str
-        """        
+        """
         datastream_name = DSUtil.get_datastream_name(dataset)
         start_date, start_time = DSUtil.get_start_time(dataset)
         return f"{datastream_name}.{start_date}.{start_time}{file_extension}"
@@ -520,17 +307,19 @@ class DSUtil:
         :type raw_dataset: xr.Dataset
         :param old_filename: The name of the original raw file.
         :type old_filename: str
-        :param config: The Config object used to assist reading time data from 
+        :param config: The Config object used to assist reading time data from
             the raw_dataset.
         :type config: Config
         :return: The standardized filename of the raw file.
         :rtype: str
-        """        
+        """
         original_filename = os.path.basename(old_filename)
         raw_datastream_name = config.pipeline_definition.input_datastream_name
-        time_var = config.dataset_definition.get_variable('time')
+        time_var = config.dataset_definition.get_variable("time")
         start_date, start_time = DSUtil.get_raw_start_time(raw_dataset, time_var)
-        return f"{raw_datastream_name}.{start_date}.{start_time}.raw.{original_filename}"
+        return (
+            f"{raw_datastream_name}.{start_date}.{start_time}.raw.{original_filename}"
+        )
 
     @staticmethod
     def get_date_from_filename(filename: str) -> str:
@@ -541,7 +330,7 @@ class DSUtil:
         :type filename: str
         :return: The date, in "yyyymmdd.hhmmss" format.
         :rtype: str
-        """     
+        """
         filename = os.path.basename(filename)
         date = filename.split(".")[3]
         time = filename.split(".")[4]
@@ -552,21 +341,18 @@ class DSUtil:
         """Given a filename that conforms to MHKiT-Cloud Data Standards, return
         the datastream name.  Datastream name is everything to the left of the
         third '.' in the filename.
-        
+
         e.g., humboldt_ca.buoy_data.b1.20210120.000000.nc
 
         :param filename: The filename or path to the file.
         :type filename: str
         :return: The datstream name, or None if filename is not in proper format.
         :rtype: Optional[str]
-        """        
-        datastream_name = None
-
-        parts = filename.split(".")
-        if len(parts) > 2:
-            datastream_name = f'{parts[0]}.{parts[1]}.{parts[2]}'
-
-        return datastream_name
+        """
+        basename = os.path.basename(filename)
+        assert len(basename.split(".")) >= 3
+        components = basename.split(".")
+        return f"{components[0]}.{components[1]}.{components[2]}"
 
     @staticmethod
     def get_datastream_directory(datastream_name: str, root: str = "") -> str:
@@ -582,7 +368,7 @@ class DSUtil:
         :type root: str, optional
         :return: The path to the directory where the datastream should be located.
         :rtype: str
-        """        
+        """
         location_id = datastream_name.split(".")[0]
         return os.path.join(root, location_id, datastream_name)
 
@@ -596,15 +382,12 @@ class DSUtil:
         :return: True if the file extension matches an image mimetype
         :rtype: bool
         """
-        is_an_image = False
+        import mimetypes
+
+        mimetypes.init()
+
         mimetype = mimetypes.guess_type(filename)[0]
-        if mimetype is not None:
-            mimetype = mimetype.split('/')[0]
-            if mimetype == 'image':
-                is_an_image = True
+        return mimetype and mimetype.split("/")[0] == "image"
 
-        return is_an_image
 
 # TODO: Maybe we need a method to be able to quickly dump out a summary of the list of problems with the data.
-
-
